// Copyright (c) Caliper Corporation 2003.  All Rights Reserved
// Written by Stewart Berry - Caliper Corporation - 2003

    // This macro demonstrates how to create a map of the world with the USA on the 
    // central meridian for visual purposes
    
    // The macro illustrates:
    // 1. How to use the area and line layer conversion functions
    // 2. How to use the ChooseFile(), ChooseFileName() and GetTempFile() functions
    // 3. Various usage of coordinates: in adding links, 
    //    and in map projections and scopes
    // 4. How to use the AddLayerToWorkspace() and DropLayerFromWorkspace() functions
    // 5. How to add Cancel buttons to progress bars created by GISDK functions
    // 6. How to use other functions: GetDBInfo(), GetDBLayers(), SetMapRedraw(), GetLayerType()
    // 7. How to use Macro "G30 new layer default settings"

    // To run this macro using the GISDK toolbox:
    // 1. Choose Tools-Add-Ins and open the GIS Developer's Kit toolbox.
    // 2. Compile this macro in test mode using the first button (Compile).
    // 3. Run this macro by clicking the second button (Test), typing the macro 
    //    name "ReprojectUSA", clicking the Macro radio button, and clicking OK.
    // N.B. A good geographic file to use this macro with is in the default ccdata 
    //      directory, and is named CCWORLDL.CDF

Macro "ReprojectUSA"
    
    // Open an area layer showing the world and add it to the workspace (not a map)
    on Escape do // Special error handling
        on escape default // Default error handling
        return() 
    end 
    // Allow the user to choose a geographic file (default directory is ccdata)
    wld_lyr =    ChooseFile({
	                        {"Compact", "*.cdf"},
	                        {"Standard","*.dbd"}},
	                         "Choose an Area Geographic File of the World (e.g. CCWORLDL.CDF)", 
	                         {{"Initial Directory", "c:\\ccdata\\"}})
    on Escape default // Default error handling
    // Get information on the "database"/"geographic file" and add layer to the workspace
    info = GetDBInfo(wld_lyr)
    layers = GetDBLayers(wld_lyr)
    layer_name = AddLayerToWorkspace(, wld_lyr, layers[1])
    
    // Check to see if layer is an area layer and if not then drop layer from workspace
    layer_type = GetLayerType(layer_name)
    if layer_type <> "Area" then do
        ShowMessage("This is not an area layer.")
        DropLayerFromWorkspace(layer_name)
        return()
    end
    
    // Convert to lines
    geo_file = GetTempFileName("*.dbd") // Get a temporary file name
    // Special error handling to add a Cancel button to the progress bar
    // created by the GISDK function
    on escape do
        on escape default // Default error handling
        DropLayerFromWorkspace(layer_name) // If user clicked cancel, drop layer from workspace
        return()
    end
    ConvertToLineDatabase(layer_name, geo_file, {{"Label", "layer"},
	                                             {"Layer Name", "layer"}})
    on escape default // Default error handling

    // Add line layer to workspace and set as the working/current layer
    actual_lyr_name = AddLayerToWorkspace(, geo_file, "layer")
    setlayer(actual_lyr_name)

    // Add links that intersect the line layer at specific locations:
    // This allows the creation of a topological area layer with the USA at central meridian
    AddLink({Coord(90000000, -89999999),Coord(90000000, 89999999)}, ,)
    AddLink({Coord(90000001, -89999999),Coord(90000001, 89999999)}, ,)
    
    // Convert to areas
    // If the user cancels either when choosing a file name or during conversion:
    on escape do // Special error handling
        on escape default // Default error handling
        DropLayerFromWorkspace(layer_name)
        DropLayerFromWorkspace(actual_lyr_name)
        return()
    end
    // Allow the user to create a geographic file name (a suggested file name is provided)
    geo_file = ChooseFileName({{"Standard","*.dbd"}}, "Save New Reprojected Layer As", 
                              {{"Suggested Name", "ReprojectedUSA.dbd"}}  )
    ConvertToAreaDatabase(actual_lyr_name, geo_file, {{"Label", "Reprojected layer"},
	                                             {"Layer Name", "Reprojected layer"}})
    on escape default // Default error handling
    
    // Drop unecessary layers from the workspace
    DropLayerFromWorkspace(layer_name)
    DropLayerFromWorkspace(actual_lyr_name)

    // Create map
    map_name = CreateMap("Reprojected USA Map", {{"Scope", info[1]}})
    // Prevent the map from redrawing
    SetMapRedraw(map_name, "False")
    // Set the current map, add the area layer, and set the layer as the working layer
    setmap(map_name)
    actual_lyr_name2 = AddLayer(, , geo_file, "Reprojected layer")
    // This documented GIDSK Standard UI macro *must* be called after
    // calling AddLayer() to get good behavior from a map and the style dbox
    RunMacro("G30 new layer default settings", actual_lyr_name2)
    setlayer(actual_lyr_name2)
    
    // Turn off autoprojections
    SetMapOptions(, {{"Auto Project", "False"}})
    // Change projection to have the USA at central meridian
    SetMapProjection(, "gall", {"lon_0=-90.000000", "lat_0=0"})
    // Allow the map to redraw
    SetMapRedraw(map_name, "True")

    // Change the map scope to have the USA at central meridian
    scp = Scope(Coord(-85534732, 7142951), 1570.6592222574068, 11109.598908000975, 0)
    setmapscope(,scp)

EndMacro